/* Hash table definitions */

/* This function can get called for each element in the hash table */
typedef void (*HTForEachProc)(void *record);

/* This function is used to copy record2 to record1 for lenght bytes,
   default=memcpy() */
typedef void (*HTCopyProc)(void *record1,void *record2,int length);

/* This function is used to free a record, default is free() */
typedef void (*HTFreeProc)(void *record);

/* This function is used to compare key to key2 for length bytes,
   default=memcmp() */
typedef int (*HTCompareProc)(void *key1,void *key2,int length);

/* This is the hash function, it computes a hash value les than buckets
   for the specified key, for len bytes */
typedef unsigned (*HTHashProc)(unsigned char *key,int len,unsigned buckets);

typedef struct {
		 int entries;
		 int allocated;
		 void **data;
	       } HashTableEntry;

typedef struct {
		 int buckets;
		 int key_size;
		 int rec_size;
		 HashTableEntry *data;
		 HTHashProc hash_proc;
		 HTCopyProc copy_proc;
		 HTFreeProc free_proc;
		 HTCompareProc compare_proc;
	       } HashTable;


/* Creates a Hash Table
 * Parameters:
 *   buckets:   The number of buckets, should be about the expexted 
 *              number of records in the hashtable
 *   key_size:  The size (in bytes) of the key value of a record.
 *              The key of a record must be at byte offset 0.
 *              (e.g. the first element of a struct)
 *   
 *   rec_size:  The total size of one record (in bytes)   
 *   hash_proc: Your own hash function, if NULL, 
 *              then the default hash function is used 
 *   copy_proc: Your own copy function, if NULL, 
 *              then the default copy function is used 
 *   free_proc: Your own free function, if NULL, 
 *              then the default free function is used 
 *   compare_proc: Your own compare function, if NULL, 
 *              then the default compare function is used 
 * Return value: the hash table
*/
HashTable HTCreate(int buckets,int key_size,int rec_size,
                   HTHashProc hash_proc,
                   HTCopyProc copy_proc,
                   HTFreeProc free_proc,
                   HTCompareProc compare_proc);

/* Adds a record to a Hash Table
 * Parameters:
 *   ht:     The hash table
 *   record: A pointer to the record to be added
 * Return value: none
 * Remarks: This function does not check for duplicate values!
*/
void HTAddRecord(HashTable ht,void *record);

/* Checks if a certain record exists in the hashtable
 * Parameters:
 *   ht:  The hash table
 *   key: A pointer to a key value
 * Return value: 1 if such a record exists
 *               0 if no such record exists
*/
int HTLookup(HashTable ht,void *key);

/* Returns a certain record
 * Parameters:
 *   ht:     The hash table
 *   key:    A pointer to a key value
 *   record: A pointer to a record, this will be filled by the function
 * Return value: 1 if such a record exists
 *               0 if no such record exists
*/
int HTGetRecord(HashTable ht,void *key,void *record);

/* Deletes a certain record
 * Parameters:
 *   ht:  The hash table
 *   key: A pointer to a key value
 * Return value: 1 if such a record exists
 *               0 if no such record exists
*/
int HTDeleteRecord(HashTable ht,void *key);

/* Destroys a hash table
 * Parameters:
 *   ht:  The hash table
 * Return value: none
*/
void HTDestroy(HashTable ht);

/* Calls a function for each element in the hash table
 * Parameters:
 *   ht:     The hash table
 *   feproc: A function pointer. This function will be called for each
             record in the hashtable
 * Return value: none
*/
void HTForEach(HashTable ht,HTForEachProc feproc);

/* Returns the number of records in the hashtable
 * Parameters:
 *   ht:  The hash table
 * Return value: The number of records in the hashtable
*/
int HTNumberOfRecords(HashTable ht);
